#include "ThreadUtilities.h"
#include <nsCOMPtr.h>
#include <nsIProxyObjectManager.h>
#include <nsServiceManagerUtils.h>

// Basically an isolation wrapper, to stop Javascript doing all the things to
// the proxy that it would otherwise try to do.
class RunnableProxyWrapper
  : public nsIRunnable
{
public:
  RunnableProxyWrapper(nsIRunnable* aProxy)
    : mProxy(aProxy)
  {
  }

  NS_DECL_ISUPPORTS
  NS_DECL_NSIRUNNABLE

private:
  nsCOMPtr<nsIRunnable> mProxy;

protected:
  ~RunnableProxyWrapper()
  {
  }
};

NS_IMPL_ISUPPORTS1(RunnableProxyWrapper, nsIRunnable);

NS_IMETHODIMP
RunnableProxyWrapper::Run()
{
  return mProxy->Run();
}

NS_IMPL_ISUPPORTS1(ThreadUtilities, nsIThreadUtilities);

NS_IMETHODIMP
ThreadUtilities::CreateProxiedRunnable
(
 nsIThread* target, nsIRunnable* realObject, nsIRunnable** proxy
)
{
  nsresult ret;
  nsCOMPtr<nsIProxyObjectManager> pom
    (do_GetService("@mozilla.org/xpcomproxy;1", &ret));
  NS_ENSURE_SUCCESS(ret, ret);

  nsCOMPtr<nsIRunnable> realProxy;

  ret = pom->GetProxyForObject(target, NS_GET_IID(nsIRunnable), realObject,
                               NS_PROXY_ASYNC, getter_AddRefs(realProxy));
  NS_ENSURE_SUCCESS(ret, ret);

  *proxy = new RunnableProxyWrapper(realProxy);
  NS_ADDREF(*proxy);

  return NS_OK;
}
